=== modified file 'Makefile.am'
--- Makefile.am	2009-07-11 19:23:04 +0000
+++ Makefile.am	2009-07-17 03:11:10 +0000
@@ -72,7 +72,7 @@
 		client/client_priv.h \
 		client/errname.h \
 		client/get_password.h \
-		client/my_readline.h \
+		client/linebuffer.h \
 		drizzled/configmake.h \
 		mystrings/decimal.h \
 		mysys/dynamic_array.h \
@@ -180,7 +180,7 @@
 		mysys/tree.cc \
 		mysys/typelib.cc
 
-client_drizzle_SOURCES=		client/drizzle.cc client/readline.cc
+client_drizzle_SOURCES=		client/drizzle.cc client/linebuffer.cc
 client_drizzle_LDADD=		${CLIENT_LDADD} ${READLINE_LIBS}
 
 client_drizzlecheck_SOURCES =	client/drizzlecheck.cc

=== modified file 'client/drizzle.cc'
--- client/drizzle.cc	2009-07-10 04:45:15 +0000
+++ client/drizzle.cc	2009-07-17 03:11:10 +0000
@@ -38,7 +38,7 @@
 #include <algorithm>
 #include <mystrings/m_ctype.h>
 #include <stdarg.h>
-#include "client/my_readline.h"
+#include "client/linebuffer.h"
 #include <signal.h>
 #include <sys/ioctl.h>
 #include <drizzled/configmake.h>
@@ -147,7 +147,7 @@
   int exit_status;
   uint32_t query_start_line;
   char *file_name;
-  LINE_BUFFER *line_buff;
+  LineBuffer *line_buff;
   bool batch,add_to_history;
 } STATUS;
 
@@ -1240,8 +1240,11 @@
 
   if (status.batch && !status.line_buff)
   {
-    status.line_buff =batch_readline_init(opt_max_input_line+512, stdin);
-    if (status.line_buff == NULL)
+    try
+    {
+      status.line_buff= new LineBuffer(opt_max_input_line+512, stdin);
+    } 
+    catch (...)
     {
       free_defaults(defaults_argv);
       my_end(0);
@@ -1341,7 +1344,8 @@
     if (!write_history(histfile_tmp))
       my_rename(histfile_tmp, histfile, MYF(MY_WME));
   }
-  batch_readline_end(status.line_buff);
+  delete status.line_buff;
+  status.line_buff= 0;
 
   if (sig >= 0)
     put_info(sig ? _("Aborted") : _("Bye"), INFO_RESULT,0,0);
@@ -1693,10 +1697,17 @@
   case 'e':
     status.batch= 1;
     status.add_to_history= 0;
-    if (!status.line_buff)
-      ignore_errors= 0;                         // do it for the first -e only
-    if (!(status.line_buff= batch_readline_command(status.line_buff, argument)))
-      return 1;
+    try
+    {
+      if (!status.line_buff)
+	status.line_buff= new LineBuffer(opt_max_input_line+512,NULL);
+      status.line_buff->add_string(argument);
+    } 
+    catch (...)
+    {
+      my_end(0);
+      exit(1);
+    }
     break;
   case 'o':
     if (argument == disabled_my_option)
@@ -1847,7 +1858,10 @@
   {
     if (!interactive)
     {
-      line=batch_readline(status.line_buff);
+      if (status.line_buff)
+	line= status.line_buff->readline();
+      else
+	line= 0;
       /*
         Skip UTF8 Byte Order Marker (BOM) 0xEFBBBF.
         Editors like "notepad" put this marker in
@@ -3657,7 +3671,7 @@
 {
   char source_name[FN_REFLEN], *end;
   const char *param;
-  LINE_BUFFER *line_buff;
+  LineBuffer *line_buff;
   int error;
   STATUS old_status;
   FILE *sql_file;
@@ -3685,10 +3699,14 @@
     return put_info(buff, INFO_ERROR, 0 ,0);
   }
 
-  if (!(line_buff=batch_readline_init(opt_max_input_line+512,sql_file)))
+  try
+  {
+    line_buff= new LineBuffer(opt_max_input_line+512,sql_file);
+  }
+  catch (...)
   {
     fclose(sql_file);
-    return put_info("Can't initialize batch_readline", INFO_ERROR, 0 ,0);
+    return put_info("Can't initialize LineBuffer", INFO_ERROR, 0, 0);
   }
 
   /* Save old status */
@@ -3706,7 +3724,8 @@
   // Continue as before
   status=old_status;
   fclose(sql_file);
-  batch_readline_end(line_buff);
+  delete status.line_buff;
+  line_buff= status.line_buff= 0;
   return error;
 }
 

=== renamed file 'client/readline.cc' => 'client/linebuffer.cc'
--- client/readline.cc	2009-04-17 21:01:47 +0000
+++ client/linebuffer.cc	2009-07-17 03:11:10 +0000
@@ -18,199 +18,140 @@
 #include <drizzled/global.h>
 #include <mysys/my_sys.h>
 #include <mystrings/m_string.h>
-#include "client/my_readline.h"
+#include "client/linebuffer.h"
+
+#include <stdexcept>
 
 using namespace std;
 
-static bool init_line_buffer(LINE_BUFFER *buffer,File file,uint32_t size,
-			    uint32_t max_size);
-static bool init_line_buffer_from_string(LINE_BUFFER *buffer,char * str);
-static size_t fill_buffer(LINE_BUFFER *buffer);
-static char *intern_read_line(LINE_BUFFER *buffer,uint32_t *out_length);
-
-
-LINE_BUFFER *batch_readline_init(uint32_t max_size,FILE *file)
-{
-  LINE_BUFFER *line_buff;
-  if (!(line_buff=(LINE_BUFFER*) malloc(sizeof(*line_buff))))
-    return 0;
-  memset(line_buff, 0, sizeof(*line_buff));
-  if (init_line_buffer(line_buff,fileno(file),IO_SIZE,max_size))
-  {
-    free(line_buff);
-    return 0;
-  }
-  return line_buff;
-}
-
-
-char *batch_readline(LINE_BUFFER *line_buff)
+LineBuffer::LineBuffer(uint32_t my_max_size,FILE *my_file)
+{
+  if (file)
+    file= fileno(my_file);
+  else
+    file= NULL;
+  bufread= IO_SIZE;
+  max_size= my_max_size;
+  buffer= (char *) malloc(bufread+1);
+  if (buffer == NULL)
+    throw runtime_error("malloc failed");
+  start_of_line= end_of_line= end= buffer;
+  buffer[0]= 0;
+
+  eof= read_length= 0;
+}
+
+LineBuffer::~LineBuffer()
+{
+  free(buffer);
+}
+
+void LineBuffer::add_string(const char *str)
+{
+  uint32_t old_length= (uint32_t) (end - buffer);
+  uint32_t length= (uint32_t) strlen(str);
+  char *tmpptr= (char *) realloc(buffer,old_length+length+2);
+  if (tmpptr == NULL)
+    throw runtime_error("malloc failed");
+  
+  buffer= start_of_line= end_of_line= tmpptr;
+  end= buffer + old_length;
+  if (old_length)
+    end[-1]= ' ';
+  memcpy(end, str, length);
+  end[length]= '\n';
+  end[length+1]= 0;
+  end+= length+1;
+  eof= 1;
+  max_size= 1;
+}
+
+char *LineBuffer::readline()
 {
   char *pos;
   uint32_t out_length;
 
-  if (!(pos=intern_read_line(line_buff,&out_length)))
+  if (!(pos=internal_readline(&out_length)))
     return 0;
   if (out_length && pos[out_length-1] == '\n')
     if (--out_length && pos[out_length-1] == '\r')  /* Remove '\n' */
       out_length--;                                 /* Remove '\r' */
-  line_buff->read_length=out_length;
+  read_length=out_length;
   pos[out_length]=0;
   return pos;
 }
 
-
-void batch_readline_end(LINE_BUFFER *line_buff)
-{
-  if (line_buff)
-  {
-    free(line_buff->buffer);
-    free(line_buff);
-  }
-}
-
-
-LINE_BUFFER *batch_readline_command(LINE_BUFFER *line_buff, char * str)
-{
-  if (!line_buff)
-  {
-    if (!(line_buff=(LINE_BUFFER*) malloc(sizeof(*line_buff))))
-      return 0;
-    memset(line_buff, 0, sizeof(*line_buff));
-  }
-  if (init_line_buffer_from_string(line_buff,str))
-  {
-    free(line_buff);
-    return 0;
-  }
-  return line_buff;
-}
-
-
-/*****************************************************************************
-      Functions to handle buffered readings of lines from a stream
-******************************************************************************/
-
-static bool
-init_line_buffer(LINE_BUFFER *buffer,File file,uint32_t size,uint32_t max_buffer)
-{
-  buffer->file=file;
-  buffer->bufread=size;
-  buffer->max_size=max_buffer;
-  if (!(buffer->buffer = (char*) malloc(buffer->bufread+1)))
-    return 1;
-  buffer->end_of_line=buffer->end=buffer->buffer;
-  buffer->buffer[0]=0;				/* For easy start test */
-  return 0;
-}
-
-/*
-  init_line_buffer_from_string can be called on the same buffer
-  several times. the resulting buffer will contain a
-  concatenation of all strings separated by spaces
-*/
-static bool init_line_buffer_from_string(LINE_BUFFER *buffer,char * str)
-{
-  uint32_t old_length=(uint32_t)(buffer->end - buffer->buffer);
-  uint32_t length= (uint32_t) strlen(str);
-  char * tmpptr= (char*)realloc(buffer->buffer, old_length+length+2);
-  if (tmpptr == NULL)
-    return 1;
-  
-  buffer->buffer= buffer->start_of_line= buffer->end_of_line= tmpptr;
-  buffer->end= buffer->buffer + old_length;
-  if (old_length)
-    buffer->end[-1]=' ';
-  memcpy(buffer->end, str, length);
-  buffer->end[length]= '\n';
-  buffer->end[length+1]= 0;
-  buffer->end+= length+1;
-  buffer->eof=1;
-  buffer->max_size=1;
-  return 0;
-}
-
-
-/*
-  Fill the buffer retaining the last n bytes at the beginning of the
-  newly filled buffer (for backward context).	Returns the number of new
-  bytes read from disk.
-*/
-
-static size_t fill_buffer(LINE_BUFFER *buffer)
+char *LineBuffer::internal_readline(uint32_t *out_length)
+{
+  char *pos;
+  size_t length;
+
+  start_of_line=end_of_line;
+  for (;;)
+  {
+    pos=end_of_line;
+    while (*pos != '\n' && *pos)
+      pos++;
+    if (pos == end)
+    {
+      if ((uint32_t) (pos - start_of_line) < max_size)
+      {
+	if (!(length=fill_buffer()) || length == (size_t) -1)
+	  return(0);
+	continue;
+      }
+      pos--; /* break line here */
+    }
+    end_of_line= pos+1;
+    *out_length= (uint32_t) (pos + 1 - eof - start_of_line);
+    return(start_of_line);
+  }
+}
+
+size_t LineBuffer::fill_buffer()
 {
   size_t read_count;
-  uint32_t bufbytes= (uint32_t) (buffer->end - buffer->start_of_line);
+  uint32_t bufbytes= (uint32_t) (end - start_of_line);
 
-  if (buffer->eof)
+  if (eof)
     return 0;					/* Everything read */
 
   /* See if we need to grow the buffer. */
 
   for (;;)
   {
-    uint32_t start_offset=(uint32_t) (buffer->start_of_line - buffer->buffer);
-    read_count=(buffer->bufread - bufbytes)/IO_SIZE;
+    uint32_t start_offset=(uint32_t) (start_of_line - buffer);
+    read_count=(bufread - bufbytes)/IO_SIZE;
     if ((read_count*=IO_SIZE))
       break;
-    buffer->bufread *= 2;
-    if (!(buffer->buffer = (char*) realloc(buffer->buffer,
-                                           buffer->bufread+1)))
+    bufread *= 2;
+    if (!(buffer = (char*) realloc(buffer, bufread+1)))
       return (uint32_t) -1;
-    buffer->start_of_line=buffer->buffer+start_offset;
-    buffer->end=buffer->buffer+bufbytes;
+    start_of_line= buffer+start_offset;
+    end= buffer+bufbytes;
   }
 
   /* Shift stuff down. */
-  if (buffer->start_of_line != buffer->buffer)
+  if (start_of_line != buffer)
   {
-    memmove(buffer->buffer, buffer->start_of_line, (uint32_t) bufbytes);
-    buffer->end=buffer->buffer+bufbytes;
+    memmove(buffer, start_of_line, (uint32_t) bufbytes);
+    end= buffer+bufbytes;
   }
 
   /* Read in new stuff. */
-  if ((read_count= my_read(buffer->file, (unsigned char*) buffer->end, read_count,
+  if ((read_count= my_read(file, (unsigned char*) end, read_count,
 			   MYF(MY_WME))) == MY_FILE_ERROR)
     return (size_t) -1;
 
   /* Kludge to pretend every nonempty file ends with a newline. */
-  if (!read_count && bufbytes && buffer->end[-1] != '\n')
+  if (!read_count && bufbytes && end[-1] != '\n')
   {
-    buffer->eof = read_count = 1;
-    *buffer->end = '\n';
+    eof= read_count = 1;
+    *end= '\n';
   }
-  buffer->end_of_line=(buffer->start_of_line=buffer->buffer)+bufbytes;
-  buffer->end+=read_count;
-  *buffer->end=0;				/* Sentinel */
+  end_of_line= (start_of_line=buffer)+bufbytes;
+  end+= read_count;
+  *end= 0;				/* Sentinel */
   return read_count;
 }
 
-
-
-char *intern_read_line(LINE_BUFFER *buffer,uint32_t *out_length)
-{
-  char *pos;
-  size_t length;
-
-
-  buffer->start_of_line=buffer->end_of_line;
-  for (;;)
-  {
-    pos=buffer->end_of_line;
-    while (*pos != '\n' && *pos)
-      pos++;
-    if (pos == buffer->end)
-    {
-      if ((uint32_t) (pos - buffer->start_of_line) < buffer->max_size)
-      {
-	if (!(length=fill_buffer(buffer)) || length == (size_t) -1)
-	  return(0);
-	continue;
-      }
-      pos--;					/* break line here */
-    }
-    buffer->end_of_line=pos+1;
-    *out_length=(uint32_t) (pos + 1 - buffer->eof - buffer->start_of_line);
-    return(buffer->start_of_line);
-  }
-}

=== renamed file 'client/my_readline.h' => 'client/linebuffer.h'
--- client/my_readline.h	2009-02-20 22:48:37 +0000
+++ client/linebuffer.h	2009-07-17 03:11:10 +0000
@@ -13,9 +13,21 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+#ifndef CLIENT_LINEBUFFER_H
+#define CLIENT_LINEBUFFER_H
 
-typedef struct st_line_buffer
+class LineBuffer
 {
+public:
+  LineBuffer(uint32_t max_size,FILE *file);
+  ~LineBuffer();
+
+  void add_string(const char *argument);
+  char *readline();
+private:
+  char *internal_readline(uint32_t *out_length);
+  size_t fill_buffer();
+
   File file;
   char *buffer;			/* The buffer itself, grown as needed. */
   char *end;			/* Pointer at buffer end */
@@ -24,9 +36,6 @@
   uint32_t eof;
   uint32_t max_size;
   uint32_t read_length;		/* Length of last read string */
-} LINE_BUFFER;
+};
 
-extern LINE_BUFFER *batch_readline_init(uint32_t max_size,FILE *file);
-extern LINE_BUFFER *batch_readline_command(LINE_BUFFER *buffer, char * str);
-extern char *batch_readline(LINE_BUFFER *buffer);
-extern void batch_readline_end(LINE_BUFFER *buffer);
+#endif /* CLIENT_LINEBUFFER_H */

